package Process;
######################################################################
# (c) Michael Schilli 1998
######################################################################

######################################################################
# $proc_obj=Process->new();        Konstruktor
######################################################################
sub new { 
  bless {};
}

######################################################################
# $ret = $proc_obj->start("prg");  Shell-Proze im Hintergrund starten
# $ret = $proc_obj->start(\&func); Funktion im Hintergrund starten
######################################################################
sub start {
  my ($self, $func) = @_;
                                        
  $SIG{'CHLD'} = sub { wait };         # Zombies verhindern

  $self->{'pid'} = fork();             # Subproze mit fork erzeugen

  if(!defined $self->{'pid'}) { 
      return 0;                        # Forkfehler
  } elsif($self->{'pid'} == 0) {       # Child-Proze:
      if(ref($func) eq "CODE") {
          &$func; exit 0;              #   Subroutine starten
      } else {
          exec "$func";                #   Shell-Proze starten
      }
  } else {                             # Parent-Proze:
      return 1;                        #   OK zurckgeben
  }
}

######################################################################
# $ret = $proc_obj->poll();   Prozestatus abfragen: 
#                             1="luft" 0="luft nicht"
######################################################################
sub poll {
  my $self = shift;

  exists $self->{'pid'} &&             # pid initialisiert und ...
    kill(0, $self->{'pid'});           # Proze reagiert
}

######################################################################
# $ret = $proc_obj->kill([SIGXXX]);   Signal an Proze schicken, 
#                                     Default-Parameter: SIGTERM
######################################################################
sub kill { 
  my ($self, $sig) = @_;

  $sig = 'SIGTERM' unless defined $sig;  # Falls kein Parameter spe-
                                         # zifiziert => SIGTERM-Signal

  return 0 if !exists $self->{'pid'};  # Proze initialisiert?

  kill($sig, $self->{'pid'}) || return 0;       # Signal senden

  delete $self->{'pid'};               # Proze-Variable lschen

  1;                                   # OK
}

1;
